function Welfres = AssessWelf
% ASSESSWELF Decomposes welfare changes for all models

load('models.mat');

sext0 = [0 1 0 0];

[W0,Expenditure0,PB0,PC0,PP0,SsT0,SsC0,SgT0,SgC0,~,~,Wbis0]     = WelfAll(without.model, ...
                                                  without.interp,without.s, ...
                                                  without.x,without.z,sext0);
[W1,Expenditure1,PB1,PC1,PP1,SsT1,SsC1,SgT1,SgC1,EV1,EVa1,Wbis1] = WelfAll(full.model, ...
                                                  full.interp,full.s, ...
                                                  full.x,full.z,sext0);
[W2,Expenditure2,PB2,PC2,PP2,SsT2,SsC2,SgT2,SgC2,EV2,EVa2,Wbis2] = WelfAll(sto.model, ...
                                                  sto.interp,sto.s, ...
                                                  sto.x,sto.z,sext0);
[W3,Expenditure3,PB3,PC3,PP3,SsT3,SsC3,SgT3,SgC3,EV3,EVa3,Wbis3] = WelfAll(sub.model, ...
                                                  sub.interp,sub.s, ...
                                                  sub.x,sub.z,sext0);

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = without.model.params{:};
beta = 1/(1+r);


% Consumer
Welfres(1,:)  = [EV1 EV2 EV3];
Welfres(3,:)  = Expenditure0-[Expenditure1 Expenditure2 Expenditure3];
Welfres(2,:)  = Welfres(1,:)-Welfres(3,:);
% Producers
Welfres(5,:)  = [PB1 PB2 PB3]-PB0;
Welfres(6,:)  = [PC1 PC2 PC3]-PC0;
Welfres(7,:)  = [PP1 PP2 PP3]-PP0;
Welfres(4,:)  = sum(Welfres(5:7,:),1);
% Storers
Welfres(9,:)  = [SsC1 SsC2 SsC3]-SsC0;
Welfres(10,:) = [SsT1 SsT2 SsT3]-SsT0;
Welfres(8,:)  = sum(Welfres(9:10,:),1);
% Government
Welfres(12,:) = [SgC1 SgC2 SgC3]-SgC0;
Welfres(13,:) = [SgT1 SgT2 SgT3]-SgT0;
Welfres(14,:) = -([PP1 PP2 PP3]-PP0);
Welfres(11,:) = sum(Welfres(12:14,:),1);
% Social welfare
Welfres(15,:) = sum(Welfres([1 4 8 11],:),1);
Welfres(16,:) = (1-beta)*([W1 W2 W3]-W0)/w;

[(1:16)' Welfres*100]
Welfres = Welfres(:,[2:3 1])*100;

WelfOutput = sprintf('Consumers & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(1,:));
WelfOutput = [WelfOutput sprintf('\\quad Efficiency & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(2,:))];
WelfOutput = [WelfOutput sprintf('\\quad Expenditures\\tnote{*} & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(3,:))];
WelfOutput = [WelfOutput sprintf('Producers & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(4,:))];
WelfOutput = [WelfOutput sprintf('\\quad Benefits\\tnote{*} & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(5,:))];
WelfOutput = [WelfOutput sprintf('\\quad Production costs & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(6,:))];
WelfOutput = [WelfOutput sprintf('\\quad Supply control\\tnote{*} & \\mc{$-$} & %1.3f & %1.3f \\\\ \n',Welfres(7,2:3))];
WelfOutput = [WelfOutput sprintf('Storers & 0 & 0 & 0 \\\\ \n')];
WelfOutput = [WelfOutput sprintf('\\quad Transfers\\tnote{*} & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(10,:))];
WelfOutput = [WelfOutput sprintf('\\quad Storage costs & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(9,:))];
WelfOutput = [WelfOutput sprintf('Government & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(11,:))];
WelfOutput = [WelfOutput sprintf('\\quad Storage transfers\\tnote{*} & %1.3f & \\mc{$-$} & %1.3f \\\\ \n',Welfres(13,[1 3]))];
WelfOutput = [WelfOutput sprintf('\\quad Storage costs & %1.3f & \\mc{$-$} & %1.3f \\\\ \n',Welfres(12,[1 3]))];
WelfOutput = [WelfOutput sprintf('\\quad Supply control\\tnote{*} & \\mc{$-$} & %1.3f & %1.3f \\\\ \n',Welfres(14,2:3))];
WelfOutput = [WelfOutput sprintf('Total & %1.3f & %1.3f & %1.3f \\\\ \n',Welfres(15,:))];


fid = fopen('../TeX/Tables/OptSRWelfDetailed.tex','w');
fwrite(fid,WelfOutput);
fwrite(fid,'\bottomrule');
fclose(fid);


