function Extract2LaTeX
% EXTRACT2LATEX Extracts parameters in a Latex file

params = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,xi,mu,H0,k,delta,r] = params{:};

beta  = 1/(1+r);             % Time discount

vhat0 = Y^(1-eta)/(1-eta)-D0*1^(1+alpha)/(1+alpha);
rho   = eta-theta*Y^(1-eta)/vhat0;

rholim = num2str(eta-alpha/gamma,3)

fid = fopen('../TeX/Optimal_Values.tex','w');
fprintf(fid,'\\newcommand{\\OptDemandElastP}{%g} %% Own-price demand elasticity \n',alpha);
fprintf(fid,'\\newcommand{\\OptDemandElastY}{%g} %% Income demand elasticity \n',eta);
fprintf(fid,'\\newcommand{\\OptBudgetShareCent}{%g} %% Budget share \n',gamma*100);
fprintf(fid,'\\newcommand{\\OptBudgetShare}{%g} %% Budget share \n',gamma);
fprintf(fid,'\\newcommand{\\OptThetaRA}{%s} \n',num2str(theta,3));
fprintf(fid,'\\newcommand{\\OptRRA}{%g} \n',rho);
fprintf(fid,'\\newcommand{\\OptRRALimit}{%s} \n',rholim);
fprintf(fid,'\\newcommand{\\OptIncome}{%s} \n',num2str(Y,3));
fprintf(fid,'\\newcommand{\\OptDemandPar}{%s} \n',num2str(D0,2));
fprintf(fid,'\\newcommand{\\OptSupplyPar}{%s} \n',num2str(H0,2));
fprintf(fid,'\\newcommand{\\OptStorageCost}{%g} \n',k);
fprintf(fid,'\\newcommand{\\OptSupplyElast}{%g} \n',1/mu);
fprintf(fid,'\\newcommand{\\OptMuSupply}{%g} \n',mu);
fprintf(fid,'\\newcommand{\\OptBetaDiscount}{%s} \n',num2str(beta,2));
fprintf(fid,'\\newcommand{\\OptDepreciation}{%s} \n',num2str(1-delta,2));
fclose(fid);

load('Sensi.mat','Welf')
if exist('Welf')
  j = Welf(2,:)<=0.3 & Welf(1,:)~=3 & Welf(1,:)>0.5;
  i = Welf(1,:)==0.5 & Welf(2,:)==0.15;
  j = logical(i+j);
  n = size(Welf(:,j),2);

  Welf(3:6,:) = Welf(3:6,:)*100;

  outputstring = strcat(repmat('& %1.2f',1,n),' \\\\ \n');

  fid = fopen('../TeX/Tables/SensiWelf.tex','w');
  fprintf(fid,strcat('Risk aversion ($\\rho$) ',...
                     '& \\mc{%1.2g}',repmat('& \\multicolumn{3}{c}{%1.2g}',1,3),' \\\\ \n'),...
          unique(Welf(1,j)));
  fwrite(fid,'\cmidrule(lr){3-5} \cmidrule(lr){6-8} \cmidrule(l){9-11}');
  fprintf(fid,strcat('\n Budget share ($\\gamma$) ',repmat('& \\mc{%1.2g}',1,5),...
                     ' & \\mc{%1.2g\\tnote{a}}',repmat('& \\mc{%1.2g}',1,4),' \\\\ \n'),...
          Welf(2,j));
  fwrite(fid,'\midrule \multicolumn{11}{@{}l}{{\em Welfare results\/}}\\');
  fprintf(fid,strcat('\n Consumers'' gains ',outputstring),Welf(3,j));
  fprintf(fid,strcat('Producers'' gains ',outputstring),Welf(4,j));
  fprintf(fid,strcat('Government outlays ',outputstring),Welf(5,j));
  fprintf(fid,strcat('Total gains ',outputstring),Welf(6,j));

  fwrite(fid,'\addlinespace \multicolumn{11}{@{}l}{{\em Asymptotic statistics\/}}\\');
  outputstring = strcat(repmat('& \\mc{%1.3f}',1,n),' \\\\ \n');
  fprintf(fid,strcat('\n Mean price ',outputstring),Welf(7,j));
  fprintf(fid,strcat('CV of price ',outputstring),Welf(8,j));
  fwrite(fid,'\bottomrule');
  fclose(fid);
end