function EV = IdealEV(P0,rho,share,elastY,elastP,P1)
% IDEALEV Calculates the equivalent variation of passing from one price
% distribution to another

if nargin<2
  rho    = -4;               % Relative risk aversion
  share  = 0.4;              % Budget share
  elastY = 0.8;              % Income elasticity
  elastP = -.6;              % Price elasticity
  P1     = mean(P0);
end

y     = 100;              % Income

% Isoelastic demand function
s     = share;
eta   = elastY;
alpha = elastP;

k     = s*(y^(1-eta));    % Parameter of normalisation
theta = (eta-rho)*(y^(1-eta)/(1-eta)-k/(1+alpha))/(y^(1-eta));
% Parameter defining the risk aversion inside in the indirect utility
% function (value defined at the deterministic SS)
utility = @(p,Y) (Y.^(1-eta)/(1-eta)-k*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta);
% Indirect utility function
sigma   = (eta-rho)*s-alpha;
% If sigma is positive, then price destabilisation is desirable (mean
% price remaining unchanged). See Turnovsky, Shalit and Schmitz (1980)

% Consumer welfare
%% Implicit definition of welfare change
norma  = mean(utility(P0,y)-utility(P1,y));   % Normalisation of equation to 1 for EV=0
findEV = @(EV) mean(utility(P0,y+EV)-utility(P1,y))/norma;
findCV = @(CV) mean(utility(P0,y)-utility(P1,y-CV))/norma;

%% Welfare calculation (Newton-Krylov solver)
EV     = nsoli(findEV,0);
