function LongRun
% LONGRUN Generates observations for all models on the asymptotic distribution

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
load('models.mat');

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = without.model.params{:};
utility = @(p,y) (y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;


s0 = [1 0];

nrep   = 1E4;
nper   = 118;
nasymp = nrep*(nper-18);

Priceres = zeros(5,5);
PriceQuantres = zeros(5,5);
Stock    = zeros(1,5);

A        = zeros(nasymp,5);  % Availability

%% Components of the social welfare function
v        = zeros(nasymp,5);  % Utility
HProfit  = zeros(nasymp,5);  % Producer profit in the current year
SProfit  = zeros(nasymp,5);  % Storer profit in the current year
Cost     = zeros(nasymp,5);  % Government outlays

%% Benchmark
stream0.reset
[ssim,x0,esim,~,stat] = recsSimul(without.model,without.interp,repmat(s0(1),nrep,1),nper);
A(:,1) = reshape(ssim(:,:,20:end),[],1);

Stock(:,1)    = stat.moments(2,1);
Priceres(1,1) = stat.moments(4,1);
Priceres(2,1) = stat.acor(4,1);
Priceres(3,1) = stat.moments(4,2)/stat.moments(4,1);
Priceres(4,1) = stat.moments(4,3);
Priceres(5,1) = stat.moments(4,4)-3;

PriceQuantres(:,1) = quantile(reshape(x0(:,3,20:end),[],1),[0.01 0.25 0.5 0.75 0.99]);

v(:,1)       = reshape(utility(x0(:,3,20:end),Y),nasymp,1);
HProfit(:,1) = reshape(x0(:,3,20:end).*x0(:,2,19:end-1).*esim(:,1,20:end)...
                       -H0*x0(:,2,20:end).^(mu+1)/(mu+1),nasymp,1);
SProfit(:,1) = reshape(x0(:,3,20:end).*x0(:,1,19:end-1)*(1-delta)...
                       -(x0(:,3,20:end)+k).*x0(:,1,20:end),nasymp,1);

Heps = reshape(x0(:,2,19:end-1).*esim(:,1,20:end),[],1);
figure
hist(Heps*100./A(:,1),20)
xlim([80 100])
xlabel('Share of production in availability')
%set(gcf,'PaperSize',[13 11]/2)
%set(gcf, 'PaperPositionMode', 'manual');
orient landscape
%set(gcf, 'PaperPositionMode', 'landscape');
%set(gcf, 'PaperUnits', 'inches');
%set(gcf, 'PaperPosition', [0.25 0.25 12.5 10.5]/2)
print('-deps','../TeX/Figures/Times/hist-Heps-share.eps')
print('-dpdf','../TeX/Figures/Times/hist-Heps-share.pdf')
clear ssim x0 esim

%% Optimal policy with both instrument
stream0.reset
[ssim,xsim,esim,~,stat] = recsSimul(full.model,full.interp,repmat(s0,nrep,1),nper);
A(:,2) = reshape(ssim(:,1,20:end),[],1);

Stock(:,2)    = stat.moments(6,1);
Priceres(1,2) = stat.moments(5,1);
Priceres(2,2) = stat.acor(5,1);
Priceres(3,2) = stat.moments(5,2)/stat.moments(5,1);
Priceres(4,2) = stat.moments(5,3);
Priceres(5,2) = stat.moments(5,4)-3;

PriceQuantres(:,2) = quantile(reshape(xsim(:,3,20:end),[],1),[0.01 0.25 0.5 0.75 0.99]);

v(:,2)       = reshape(utility(xsim(:,3,20:end),Y),nasymp,1);
HProfit(:,2) = reshape(xsim(:,3,20:end).*xsim(:,2,19:end-1).*esim(:,1,20:end)+...
                      xsim(:,6,20:end).*xsim(:,2,20:end)-H0*xsim(:,2,20:end).^(mu+1)/(mu+1),nasymp,1);
SProfit(:,2) = reshape(xsim(:,3,20:end).*xsim(:,1,19:end-1)*(1-delta)...
                       -(xsim(:,3,20:end)+k).*xsim(:,1,20:end),nasymp,1);
Cost(:,2)    = reshape((xsim(:,3,20:end)+k).*xsim(:,4,20:end)-xsim(:,3,20:end).*xsim(:,4,19:end-1)*(1-delta)+...
                       xsim(:,6,20:end).*xsim(:,2,20:end),nasymp,1);

clear ssim xsim esim

%% Optimal public storage policy
stream0.reset
[ssim,xsim,esim,~,stat] = recsSimul(sto.model,sto.interp,repmat(s0,nrep,1),nper);
A(:,3) = reshape(ssim(:,1,20:end),[],1);

Stock(:,3)    = stat.moments(6,1);
Priceres(1,3) = stat.moments(5,1);
Priceres(2,3) = stat.acor(5,1);
Priceres(3,3) = stat.moments(5,2)/stat.moments(5,1);
Priceres(4,3) = stat.moments(5,3);
Priceres(5,3) = stat.moments(5,4)-3;

PriceQuantres(:,3) = quantile(reshape(xsim(:,3,20:end),[],1),[0.01 0.25 0.5 0.75 0.99]);

v(:,3)       = reshape(utility(xsim(:,3,20:end),Y),nasymp,1);
HProfit(:,3) = reshape(xsim(:,3,20:end).*xsim(:,2,19:end-1).*esim(:,1,20:end) ...
                      -H0*xsim(:,2,20:end).^(mu+1)/(mu+1),nasymp,1);
SProfit(:,3) = reshape(xsim(:,3,20:end).*xsim(:,1,19:end-1)*(1-delta) ...
                       -(xsim(:,3,20:end)+k).*xsim(:,1,20:end),nasymp,1);
Cost(:,3)    = reshape((xsim(:,3,20:end)+k).*xsim(:,4,20:end) ...
                       -xsim(:,3,20:end).*xsim(:,4,19:end-1)*(1-delta),nasymp,1);

clear ssim xsim esim

%% Optimal supply control subsidy
stream0.reset
[ssim,xsim,esim,~,stat] = recsSimul(sub.model,sub.interp,repmat(s0,nrep,1),nper);
A(:,4) = reshape(ssim(:,1,20:end),[],1);

Stock(:,4)    = stat.moments(3,1);
Priceres(1,4) = stat.moments(5,1);
Priceres(2,4) = stat.acor(5,1);
Priceres(3,4) = stat.moments(5,2)/stat.moments(5,1);
Priceres(4,4) = stat.moments(5,3);
Priceres(5,4) = stat.moments(5,4)-3;

PriceQuantres(:,4) = quantile(reshape(xsim(:,3,20:end),[],1),[0.01 0.25 0.5 0.75 0.99]);

v(:,4)       = reshape(utility(xsim(:,3,20:end),Y),nasymp,1);
HProfit(:,4) = reshape(xsim(:,3,20:end).*xsim(:,2,19:end-1).*esim(:,1,20:end)+...
                      xsim(:,2,20:end).*xsim(:,5,20:end)-H0*xsim(:,2,20:end).^(mu+1)/(mu+1),nasymp,1);
SProfit(:,4) = reshape(xsim(:,3,20:end).*xsim(:,1,19:end-1)*(1-delta)...
                       -(xsim(:,3,20:end)+k).*xsim(:,1,20:end),nasymp,1);
Cost(:,4)    = reshape(xsim(:,5,20:end).*xsim(:,2,20:end),nasymp,1);

clear ssim xsim esim

%% Futures
stream0.reset
[ssim,xsim,esim,~,stat] = recsSimul(futures.model,futures.interp, ...
                                    repmat([s0(1) 1 0],nrep,1),nper);
A(:,5) = reshape(ssim(:,1,20:end),[],1);

Stock(:,5)    = stat.moments(4,1);
Priceres(1,5) = stat.moments(6,1);
Priceres(2,5) = stat.acor(6,1);
Priceres(3,5) = stat.moments(6,2)/stat.moments(6,1);
Priceres(4,5) = stat.moments(6,3);
Priceres(5,5) = stat.moments(6,4)-3;

PriceQuantres(:,5) = quantile(reshape(xsim(:,3,20:end),[],1),[0.01 0.25 0.5 0.75 0.99]);

v(:,5)       = reshape(utility(xsim(:,3,20:end),Y+(xsim(:,3,20:end)-ssim(:,2,20:end)).*ssim(:,3,20:end)),nasymp,1);
HProfit(:,5) = reshape(xsim(:,3,20:end).*xsim(:,2,19:end-1).*esim(:,1,20:end)-...
                       H0*xsim(:,2,20:end).^(mu+1)/(mu+1),nasymp,1);
SProfit(:,5) = reshape(xsim(:,3,20:end).*xsim(:,1,19:end-1)*(1-delta)...
                       -(xsim(:,3,20:end)+k).*xsim(:,1,20:end),nasymp,1);
Cost(:,5)    = reshape(-(xsim(:,3,20:end)-ssim(:,2,20:end)).*ssim(:,3,20:end),nasymp,1);

clear ssim xsim esim

%%
save('LongRun.mat','A','-v6');

Stock

Priceres
PriceQuantres

disp('Standard deviation of utility');
std(v)

disp('Producer profit: mean, standard deviation and CV');
disp(mean(HProfit))
disp(std(HProfit))
disp(std(HProfit)./mean(HProfit))

disp('Storer profit: mean and standard deviation');
disp(mean(SProfit))
disp(std(SProfit))

disp('Government outlays: mean and standard deviation');
disp(mean(Cost))
disp(std(Cost))

figure()
subplot(2,2,1)
plot(A(:,1),HProfit(:,1),'.')
subplot(2,2,2)
plot(A(:,2),HProfit(:,2),'.')
subplot(2,2,3)
plot(A(:,3),HProfit(:,3),'.')
subplot(2,2,4)
plot(A(:,4),HProfit(:,4),'.')

NewOrder = [1 3:4 2 5];

RiskSharing = sprintf('Consumers utility & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',std(v(:,NewOrder(1:4))));
RiskSharing = [RiskSharing sprintf('Producers profit & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',std(HProfit(:,NewOrder(1:4))))];
RiskSharing = [RiskSharing sprintf('Storers profit & %1.3f & 0 & %1.3f & 0 \\\\ \n',std(SProfit(:,[1 4])))];
RiskSharing = [RiskSharing sprintf('Government outlays & 0 & %1.3f & %1.3f & %1.3f \\\\ \n',std(Cost(:,[3:4 2])))];

fid = fopen('../TeX/Tables/RiskSharing.tex','w');
fwrite(fid,RiskSharing);
fwrite(fid,'\bottomrule');
fclose(fid);

PriceStOutput = sprintf('\\quad Mean & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',Priceres(1,NewOrder));
PriceStOutput = [PriceStOutput sprintf('\\quad Coefficient of variation & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',Priceres(3,NewOrder))];
PriceStOutput = [PriceStOutput sprintf('\\quad Skewness & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',Priceres(4,:))];

PriceStOutput

PriceQtOutput = sprintf('\\qquad 1\\%% & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',PriceQuantres(1,NewOrder));
PriceQtOutput = [PriceQtOutput sprintf('\\qquad 25\\%% & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',PriceQuantres(2,NewOrder))];
PriceQtOutput = [PriceQtOutput sprintf('\\qquad 50\\%% & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',PriceQuantres(3,NewOrder))];
PriceQtOutput = [PriceQtOutput sprintf('\\qquad 75\\%% & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',PriceQuantres(4,NewOrder))];
PriceQtOutput = [PriceQtOutput sprintf('\\qquad 99\\%% & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',PriceQuantres(5,NewOrder))];

fid = fopen('../TeX/Tables/OptLRStat.tex','w');
fwrite(fid,'Price&&&&& \\');
fwrite(fid,PriceStOutput);
fwrite(fid,'\quad Quantiles&&&&& \\');
fwrite(fid,PriceQtOutput);
fprintf(fid,'\\addlinespace \n Mean stock level & %1.3f & %1.3f & %1.3f & %1.3f & %1.3f \\\\ \n',Stock(:,NewOrder));
fwrite(fid,'\bottomrule');
fclose(fid);

PriceQtOutput
