function RiskPremium
% RISKPREMIUM Calculates changes in risk-premium

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
load('models.mat');

params = without.model.params;
[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
beta = 1/(1+r);
utility = @(p,y) (y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;

sext0 = [0 1 0 0];
s0    = [sum(sext0(1:3)) sext0(4)];

nrep = 15000;
nper = 150;
ssim = cell(4,1);
xsim = cell(4,1);
BETA = beta.^(0:(nper));
ERiskPremium = zeros(3,4);

stream0.reset
[ssim{1},xsim{1}] = recsSimul(without.model,without.interp,repmat(s0(1),nrep,1),nper);

stream0.reset
[ssim{2},xsim{2}] = recsSimul(full.model,full.interp,repmat(s0,nrep,1),nper);

stream0.reset
[ssim{3},xsim{3}] = recsSimul(sto.model,sto.interp,repmat(s0,nrep,1),nper);

stream0.reset
[ssim{4},xsim{4}] = recsSimul(sub.model,sub.interp,repmat(s0,nrep,1),nper);

for i=1:4
  A  = squeeze(ssim{i}(:,1,:))';
  Ss = squeeze(xsim{i}(:,1,:))';
  H  = squeeze(xsim{i}(:,2,:))';
  P  = squeeze(xsim{i}(:,3,:))';
  switch i
   case {1,4}
    Sg = zeros(size(Ss));
   case {2,3}
    Sg = squeeze(xsim{i}(:,4,:))';
  end

  % Risk-premium of the consumer with respect to a stabilisation at mean price
  Ev = BETA*utility(mean(P,2),Y);
  [tmp,~,exitflag] = fzero(@(RP) BETA*mean(utility(P,Y+RP),2)-Ev,0.001);
  if exitflag==1
    ERiskPremium(1,i) = tmp;
  else
    error('Failure to converge in fzero');
  end

  % Risk-premium of the consumer with respect to a stabilisation at steady-state price
  Ev = BETA*utility(ones(nper+1,1),Y);
  [tmp,~,exitflag] = fzero(@(RP) BETA*mean(utility(P,Y+RP),2)-Ev,0.001);
  if exitflag==1
    ERiskPremium(2,i) = tmp;
  else
    error('Failure to converge in fzero');
  end

  % Risk-premium of the whole economy
  EW = BETA*(utility(ones(nper+1,1),Y)+w*(ones(nper+1,1)-H0*ones(nper+1,1).^(1+mu)/(1+mu)));
  [tmp,~,exitflag] = fzero(@(RP) ...
                           BETA*mean(utility(P,Y+RP)+w*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*(Ss+Sg)),2)-EW,0.001);
  if exitflag==1
    ERiskPremium(3,i) = tmp;
  else
    error('Failure to converge in fzero');
  end
end

dRP = (ERiskPremium(:,ones(1,3))-ERiskPremium(:,2:end))*100;

disp('Risk premium')
disp(ERiskPremium*100)
disp('Change in risk premium')
disp(dRP)

save('models.mat','ERiskPremium','dRP','-append');