function res = SensiDemand
% SENSIDEMAND Analyses sensitivity to price and income elasticity

delete('Params.mat');

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);

sext0 = [0 1 0 0];
s0    = [sum(sext0(1:3)) sext0(4)];
nrep  = 1E4;
nper  = 118;

ElastP = [-0.2 -0.4 -0.7];
ElastY = [0.05 0.25 0.5];

res = zeros(10,length(ElastP)*length(ElastY));

l = 0;
for i=1:length(ElastP)
  for j=1:length(ElastY)
    l = l+1;
    alpha = ElastP(i);
    eta   = ElastY(j);
    [l alpha eta]
    save('Params.mat','alpha','eta');
    res(1:2,l) = [alpha eta];

    [model,interp,s,x,z] = runmodel(0);
    [W0,EPS0,ESS0] = Welfmodel(model,interp,s,x,z,sext0(1:2))
    stream0.reset
    [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0(1),nrep,1),nper);
    res(5,l)       = stat.moments(4,2)/stat.moments(4,1);
    res(8,l)       = stat.moments(2,1);

    [model,interp,s,x,z] = runcommitmentfull(0);
    [W1,EPS1,ECost1,ESS1,EV1] = Welffull(model,interp,s,x,z,sext0)
    stream0.reset
    [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
    res(3,l)  = EV1+EPS1-EPS0+ESS1-ESS0-ECost1;
    res(6,l)  = stat.moments(5,2)/stat.moments(5,1);
    res(9,l)  = stat.moments(6,1);

    [model,interp,s,x,z] = runcommitmentsto(0);
    [W2,EPS2,ECost2,ESS2,EV2] = Welfsto(model,interp,s,x,z,sext0)
    stream0.reset
    [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
    res(4,l) = EV2+EPS2-EPS0+ESS2-ESS0-ECost2;
    res(7,l) = stat.moments(5,2)/stat.moments(5,1);
    res(10,l)  = stat.moments(6,1);
    close all
  end
end

delete('Params.mat');

Demand = res;
save('Sensi.mat','Demand','-append');