function res = SensiDiscount
% SENSIDISCOUNT Analyses the sensitivity to the discount parameter

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
delete('Params.mat');

sext0 = [0 1 0 0];
s0    = [sum(sext0(1:3)) sext0(4)];
nrep  = 1E4;
nper  = 118;

Interest = [0.005 0.01 0.02 0.05 0.1];

res = zeros(7,length(Interest));

l = 0;
for j=1:length(Interest)
  l = l+1;
  res(1,l) = Interest(j);
  r = Interest(j);
  save('Params.mat','r');

  [model,interp,s,x,z] = runmodel(0);
  [W0,EPS0,ESS0] = Welfmodel(model,interp,s,x,z,sext0(1:2))
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0(1),nrep,1),nper);
  res(4,l)       = stat.moments(4,2)/stat.moments(4,1); % Price CV

  [model,interp,s,x,z] = runcommitmentfull(0);
  [W1,EPS1,ECost1,ESS1,EV1] = Welffull(model,interp,s,x,z,sext0)
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
  res(2,l) = EV1+EPS1-EPS0+ESS1-ESS0-ECost1; % Total gains
  res(5,l) = stat.moments(5,2)/stat.moments(5,1); % Price CV

  [model,interp,s,x,z] = runcommitmentsto(0);
  [W2,EPS2,ECost2,ESS2,EV2] = Welfsto(model,interp,s,x,z,sext0)
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
  res(3,l) = EV2+EPS2-EPS0+ESS2-ESS0-ECost2; % Total gains
  res(6,l) = stat.moments(5,2)/stat.moments(5,1); % Price CV
  res(7,l) = EPS2-EPS0; % Producer gains

  close all
end

delete('Params.mat');

Discount = res;
save('Sensi.mat','Discount','-append');