function res = SensiInitPt
% SENSIINITPT Analyses sensitivity to initial state variables

load('models.mat');

State = [0 0.8;0 1; 0 1.2];

res = zeros(size(State,1),21);

for i=1:size(State,1)
  res(i,1:3) = [State(i,:) sum(State(i,:),2)];

  [W0,EPS0,ESS0,PA0] = Welfmodel(without.model,without.interp,without.s,without.x,without.z,State(i,:));
  [W1,EPS1,ECost1,ESS1,EV1,~,~,~,PA1] = Welffull(full.model,full.interp,full.s,full.x,full.z,[State(i,:) 0 0]);
  res(i,4:9) = [EV1 EPS1-EPS0 ESS1-ESS0 PA1-PA0 -ECost1 EV1+EPS1+ESS1-ECost1-EPS0-ESS0];

  [W2,EPS2,ECost2,ESS2,EV2,~,PA2] = Welfsto(sto.model,sto.interp,sto.s,sto.x,sto.z,[State(i,:) 0 0]);
  res(i,10:15) = [EV2 EPS2-EPS0 ESS2-ESS0 PA2-PA0 -ECost2 EV2+EPS2+ESS2-ECost2-EPS0-ESS0];

  [W3,EPS3,ECost3,ESS3,EV3,~,PA3] = Welfsub(sub.model,sub.interp,sub.s,sub.x,sub.z,[State(i,:) 0 0]);
  res(i,16:21) = [EV3 EPS3-EPS0 ESS3-ESS0 PA3-PA0 -ECost3 EV3+EPS3+ESS3-ECost3-EPS0-ESS0];
end

res(:,4:end) = res(:,4:end)*100;

outputstring = strcat(repmat('%1.2f & ',1,8),' %1.2f \\\\ \n');

fid = fopen('../TeX/Tables/OptSensiInit.tex','w');
fprintf(fid,strcat('Consumers'' gains &&',outputstring),res(:,[10 16 4]));
fprintf(fid,strcat('Producers'' gains &&',outputstring),res(:,[11 17 5]));
fprintf(fid,strcat('\\quad from initial availability &&',outputstring),res(:,[13 19 7]));
fprintf(fid,strcat('Government outlays &&',outputstring),-res(:,[14 20 8]));
fprintf(fid,strcat('Total gains &&',outputstring),res(:,[15 21 9]));
fwrite(fid,'\bottomrule');
fclose(fid);
