function SensiLinearDemand
% SENSILINEARDEMAND Solves the benchmark model and the optimal policy with a linear demand function

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
s0      = 1;
nrep    = 1E4;
nper    = 118;

% GET parameters
[params,shocks,interp] = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

% PACK MODEL STRUCTURE
model.func   = 'mmodellinear';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

xinit     = [max(zeros(n,1),0.8*(s-1)) ones(n,1) max((s/(D0*Y^eta)).^(1/alpha),0.7)];
interp.cz = ones(n,2);
interp.cx = xinit;
options = struct(...
    'display'    , 1,...
    'reesolver'  , 'mixed',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'reesolveroptions',struct('atol',1E-10,'rtol',1E-10));

recsCheck(model,1,[0 1 1],[1 1]);

tic
[interp.cx,x,z] = recsSolveREE(interp,model,s,xinit,options);
toc
interp.cz = funfitxy(interp.fspace,interp.Phi,z);

stream0.reset
[~,~,~,~,stat] = recsSimul(model,interp,repmat(s0(1),nrep,1),nper);

% Optimal policy
model1.func = 'mcommitmentfullbislinear';

Pbar = 1;
w    = gamma*(2-eta);

model1.params = {gamma,Y,alpha,eta,D0,theta,Pbar,w,mu,H0,k,delta,r};
model1        = catstruct(shocks,model1);

recsCheck(model1,1,[0 1 1 0 1],[1 1 1 1]);

interp1    = interp;
x1         = [x zeros(n,2)];
interp1.cx = funfitxy(interp1.fspace,s,x1);

tic
[interp1.cx,x1,z1] = recsSolveREE(interp1,model1,s,x1,options);
toc
interp1.cz = funfitxy(interp1.fspace,interp1.Phi,z1);

stream0.reset
[~,~,~,~,stat1] = recsSimul(model1,interp1,repmat(s0,nrep,1),nper);

disp('Asymptotic mean price:')
[stat.moments(4,1) stat1.moments(4,1)]

% Welfare
[W00,W10,EPS00,EPS10,ECost0,EV,ECostS0,ECostH0] = Welflinear(s,[0 1],model ,interp ,x ,z,...
                                                                     model1, ...
                                                  interp1,x1,z1)

disp(100*[EV EPS10-EPS00 ECost0 EV+EPS10-EPS00-ECost0]')