function res = SensiSupply
% SENSISUPPLY Analyses the sensitivity to supply elasticity

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
delete('Params.mat');

sext0 = [0 1 0 0];
s0    = [sum(sext0(1:3)) sext0(4)];
nrep  = 1E4;
nper  = 118;

Supply        = [0.01 0.1 0.5 1 2];

res = zeros(6,length(Supply));

l = 0;
for j=1:length(Supply)
  l = l+1;
  res(1,l) = Supply(j);
  mu = 1/Supply(j);
  save('Params.mat','mu');

  [model,interp,s,x,z] = runmodel(0);
  [W0,EPS0,ESS0] = Welfmodel(model,interp,s,x,z,sext0(1:2))
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0(1),nrep,1),nper);
  res(4,l)       = stat.moments(4,2)/stat.moments(4,1);

  [model,interp,s,x,z] = runcommitmentfull(0);
  [W1,EPS1,ECost1,ESS1,EV1] = Welffull(model,interp,s,x,z,sext0)
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
  res(2,l) = EV1+EPS1-EPS0+ESS1-ESS0-ECost1;
  res(5,l) = stat.moments(5,2)/stat.moments(5,1);

  [model,interp,s,x,z] = runcommitmentsto(0);
  [W2,EPS2,ECost2,ESS2,EV2] = Welfsto(model,interp,s,x,z,sext0)
  stream0.reset
  [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
  res(3,l) = EV2+EPS2-EPS0+ESS2-ESS0-ECost2;
  res(6,l) = stat.moments(5,2)/stat.moments(5,1);
end

close all
delete('Params.mat');

Supply = res;
save('Sensi.mat','Supply','-append');