function res = SensiWelf
% SENSIWELF Analyses the sensitivity to risk aversion and budget share

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
delete('Params.mat');

sext0 = [0 1 0 0];
s0    = [sum(sext0(1:3)) sext0(4)];
nrep  = 1E4;
nper  = 118;

RRA   =  [0.5 1 2 3 4];
share = [0.01 0.15 0.3 0.5];

res = zeros(8,length(RRA)*length(share));

l = 0;
for i=1:length(RRA)
  for j=1:length(share)

    l = l+1;
    rho   = RRA(i)
    gamma = share(j)
    save('Params.mat','rho','gamma');
    res(1:2,l) = [rho gamma];

    [model,interp,s,x,z] = runmodel(0);
    [W0,EPS0,ESS0] = Welfmodel(model,interp,s,x,z,sext0(1:2))

    if rho==2 && gamma==0.15
      [model,interp,s,x,z] = runcommitmentfull;
    else
      [model,interp,s,x,z] = runcommitmentfull(0);
    end
    [W1,EPS1,ECost1,ESS1,EV1] = Welffull(model,interp,s,x,z,sext0)
    stream0.reset
    [~,~,~,~,stat] = recsSimul(model,interp,repmat(s0,nrep,1),nper);
    res(3:6,l) = [EV1; EPS1-EPS0; ECost1; EV1+EPS1-EPS0+ESS1-ESS0-ECost1;]
    res(7,l)   = stat.moments(5,1);
    res(8,l)   = stat.moments(5,2)/stat.moments(5,1);
    res(9,l)   = stat.moments(4,1);

    close all
  end
end

delete('Weight.mat','Params.mat');

Welf = res;
save('Sensi.mat','Welf');