function [W,Expenditure,PB,PC,PP,SsT,SsC,SgT,SgC,EV,EVa,Wbis] = ...
    WelfAll(model,interp,s,x,z,sext0)
% WELFALL Calculates each component of welfare changes

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = model.params{:};
beta = 1/(1+r);

A    = s(:,1);

Ss   = x(:,1);
H    = x(:,2);
P    = x(:,3);

s0bench = (1-delta)*sext0(1)+sext0(2);

switch model.func
  case 'mmodel'
  Sg = zeros(size(x,1),1);
  Xi = zeros(size(x,1),1);
  s0 = (1-delta)*sext0(1)+sext0(2);
 case {'mcommitmentfull','mcommitmentfullbis'}
  Sg = x(:,4);
  Xi = x(:,6);
  s0 = [(1-delta)*sum(sext0(1:2))+sext0(3) sext0(4)];
 case {'mcommitmentsto','mcommitmentstobis'}
  Sg = x(:,4);
  Xi = zeros(size(x,1),1);
  s0 = [(1-delta)*sum(sext0(1:2))+sext0(3) sext0(4)];
 case 'mcommitmentsub'
  Sg = zeros(size(x,1),1);
  Xi = x(:,5);
  s0 = [(1-delta)*sext0(1)+sext0(2) sext0(4)];
end

Ss0    = sext0(:,1);
H0eps0 = sext0(:,2);
Sg0    = sext0(:,3);
P0     = funeval(interp.cx(:,3),interp.fspace,s0);

% Social welfare
utility = @(p,y) (y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;
Wt      = utility(P,Y)+w*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*(Ss+Sg));
W       = mean(infinitesums(model,interp,s,x,s0,Wt));

% Social welfare without risk-aversion
Wbist   = -D0*P.^(1+alpha)/(1+alpha)+P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*(Ss+Sg);
Wbis    = mean(infinitesums(model,interp,s,x,s0,Wbist));

% Final expenditure
Expendituret = D0*(P.^alpha)*(Y^eta).*P;
Expenditure  = (1-beta)*mean(infinitesums(model,interp,s,x,s0,Expendituret));

% Production
%% Producers benefits
PB = (1-beta)*mean(infinitesums(model,interp,s,x,s0,beta*z(:,2).*H)+P0.*H0eps0);

%% Production costs
PC = (1-beta)*mean(infinitesums(model,interp,s,x,s0,-H0*H.^(1+mu)/(1+mu)));

%% Production policy
PP = (1-beta)*mean(infinitesums(model,interp,s,x,s0,Xi.*H));

% Storage
%% Speculative storage
%%% Storage costs
SsC = (1-beta)*mean(infinitesums(model,interp,s,x,s0,-k*Ss));

%%% Storage transfers
SsT = infinitesums(model,interp,s,x,s0,(beta*(1-delta)*z(:,1)-P).*Ss);
SsT = (1-beta)*mean(P0.*Ss0*(1-delta)+SsT);

%% Public storage
%%% Storage costs
SgC = (1-beta)*mean(infinitesums(model,interp,s,x,s0,-k*Sg));

%%% Storage transfers
SgT = infinitesums(model,interp,s,x,s0,(beta*(1-delta)*z(:,1)-P).*Sg);
SgT = (1-beta)*mean(P0.*Sg0*(1-delta)+SgT);

if nargout>=9
  %% Consumer utility after stabilisation
  V1t = utility(P,Y);
  V10 = mean(infinitesums(model,interp,s,x,s0,V1t));

  %% Consumer utility before stabilisation
  load('models.mat','without');
  Pb             = without.x(:,3); % benchamrk price
  V0t            = utility(Pb,Y*1.001);
  [~,interp.cv0] = infinitesums(without.model,...
                                without.interp,...
                                without.s,without.x,s0bench,V0t);

  EV  = nsoli(@findEV,Y*0.001);

  %% Approximation of EV through w
  V0t = utility(Pb,Y);
  [V00,interp.cv0] = infinitesums(without.model,...
                                  without.interp,...
                                  without.s,without.x,s0bench,V0t,interp.cv0);
  EVa = (1-beta)*(V10-V00)/w;
end

function Obj = findEV(X)
  V0t = utility(Pb,Y+X);
  [V00,interp.cv0] = infinitesums(without.model,...
                                  without.interp,...
                                  without.s,without.x,s0bench,V0t,interp.cv0);
  Obj = mean(V00)-V10;
end

end