function [W0,EPS0,ECost0,ESS0,EV,ECostS0,ECostH0,Transfer,PA0] = Welffull(model, ...
                                                  interp,s,x,z,s0,s0bench)
% WELFFULL Calculates welfare effects for the optimal policy

if nargin < 7, s0bench = s0(:,1:2); end

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = model.params{:};
beta = 1/(1+r);

A    = s(:,1);

S    = x(:,1);
H    = x(:,2);
P    = x(:,3);
Sg   = x(:,4);
Xi   = x(:,6);

S0   = s0(:,1);
H0eps0 = s0(:,2);
Sg0  = s0(:,3);
P0   = funeval(interp.cx(:,3),interp.fspace,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)]);

%% Social welfare
utility = @(p,y) (y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;
Wt      = utility(P,Y)+w*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*(S+Sg));
W0      = mean(infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],Wt));

%% Producer surplus
PSt  = (beta*z(:,2)+Xi).*H-H0*H.^(1+mu)/(1+mu);
EPS0 = infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],PSt);
EPS0 = (1-beta)*mean(P0.*H0eps0+EPS0);

%% Fiscal cost
CostSt  = (P+k-beta*(1-delta)*z(:,1)).*Sg;
ECostS0 = infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],CostSt)-P0.*Sg0*(1-delta);
ECostS0 = (1-beta)*mean(ECostS0);
CostHt  = Xi.*H;
ECostH0 = (1-beta)*mean(infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],CostHt));
ECost0 = ECostS0+ECostH0;

%% Storer surplus
SSt  = -(P+k-beta*(1-delta)*z(:,1)).*S;
ESS0 = infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],SSt);
ESS0 = (1-beta)*mean(P0.*S0*(1-delta)+ESS0);

%% Gains from first period grain suply
PA0  = (1-beta)*mean(P0.*(S0*(1-delta)+H0eps0));

%% Consumer utility after stabilisation
V1t = utility(P,Y);
V10 = mean(infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],V1t));

%% Consumer utility before stabilisation
load('models.mat','without');
Pb               = without.x(:,3);
V0t              = utility(Pb,Y*1.001);
[~,interp.cv0] = infinitesums(without.model,...
                              without.interp,...
                              without.s,without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),V0t);


[EV,~,exitflag] = fzero(@findEV,Y*0.001);
EV              = nsoli(@findEV,EV);
% $$$ if exitflag~=1
% $$$   EV = nsoli(@findEV,Y*0.001);
% $$$ % $$$   error('fzero cannot find EV');
% $$$ end

Expenditure0t = D0*(Pb.^alpha)*(Y^eta).*Pb;
Expenditure0  = mean(infinitesums(without.model,without.interp,without.s,...
                                  without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),Expenditure0t));
Expenditure1t = D0*(P.^alpha)*(Y^eta).*P;
Expenditure1  = mean(infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,4)],...
                                  Expenditure1t));

Transfer = (1-beta)*(Expenditure0-Expenditure1);


function Obj = findEV(X)

  V0t = utility(Pb,Y+X);
  [V00,interp.cv0] = infinitesums(without.model,...
                                  without.interp,...
                                  without.s,without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),V0t,interp.cv0);
  Obj = mean(V00)-V10;

end

end