function [W0,EPS0,ECost0,ESS0,EV,Transfer,PA0] = Welffutures(model,interp,s,x,z,s0,s0bench)
% WELFFUTURES Calculates the welfare effects of the model with futures markets

if nargin < 7, s0bench = s0(:,1:2); end

[gamma,Y,alpha,eta,D0,theta,v0,xi,mu,H0,k,delta,r] = model.params{:};
beta = 1/(1+r);

A    = s(:,1);
Fs   = s(:,2);
Qs   = s(:,3);

S    = x(:,1);
H    = x(:,2);
P    = x(:,3);
F    = x(:,4);
Q    = x(:,5);

S0   = s0(:,1);
H0eps0 = s0(:,2);
P0   = funeval(interp.cx(:,3),interp.fspace,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)]);

%% Social welfare
Wt      = utility(P,Y+(P-Fs).*Qs)+xi*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*S-(P-Fs).*Qs);
W0      = mean(infinitesums(model,interp,s,x,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)],Wt));

%% Producer surplus
PSt  = beta*z(:,2).*H-H0*H.^(1+mu)/(1+mu);
EPS0 = infinitesums(model,interp,s,x,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)],PSt);
EPS0 = (1-beta)*mean(P0.*H0eps0+EPS0);

%% Fiscal cost
Costt  = (P-Fs).*Qs;
ECost0 = infinitesums(model,interp,s,x,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)],Costt)*(1-delta);
ECost0 = (1-beta)*mean(ECost0);

%% Storer surplus
SSt  = -(P+k-beta*(1-delta)*z(:,1)).*S;
ESS0 = infinitesums(model,interp,s,x,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)],SSt);
ESS0 = (1-beta)*mean(P0.*S0*(1-delta)+ESS0);

%% Gains from first period grain suply
PA0  = (1-beta)*mean(P0.*(S0*(1-delta)+H0eps0));

%% Consumer utility after stabilisation
V1t = utility(P,Y+(P-Fs).*Qs);
V10 = mean(infinitesums(model,interp,s,x,[(1-delta)*s0(:,1)+s0(:,2) s0(:,3) s0(:,4)],V1t));

%% Consumer utility before stabilisation
load('models.mat','without');
Pb               = without.x(:,3);
V0t              = utility(Pb,Y*1.001);
[~,interp.cv0] = infinitesums(without.model,...
                              without.interp,...
                              without.s,without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),V0t);

EV = nsoli(@findEV,Y*0.001);

Expenditure0t = D0*(Pb.^alpha)*(Y^eta).*Pb;
Expenditure0  = mean(infinitesums(without.model,without.interp,without.s,...
                                  without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),Expenditure0t));
Expenditure1t = D0*(P.^alpha)*(Y^eta).*P;
Expenditure1  = mean(infinitesums(model,interp,s,x,[(1-delta)*(s0(:,1)+s0(:,3))+s0(:,2) s0(:,3) s0(:,4)],...
                                  Expenditure1t));

Transfer = (1-beta)*(Expenditure0-Expenditure1);

function Obj = findEV(X)

  V0t = utility(Pb,Y+X);
  [V00,interp.cv0] = infinitesums(without.model,...
                                  without.interp,...
                                  without.s,without.x,(1-delta)*s0bench(:,1)+s0bench(:,2),V0t,interp.cv0);
  Obj = mean(V00)-V10;

end

function V = utility(p,y)

  vhat = y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha);
  
  V    = (vhat.^(1+theta)/(1+theta))/v0;
    
end

end