function [W00,W10,EPS00,EPS10,ECost0,EV,ECostS0,ECostH0] = Welflinear(s,s0,model0,interp0,x0,z0,...
                                                                           model1,interp1,x1,z1)
% WELFLINEAR Calculates welfare effects for the model with linear demand function

[gamma,Y,alpha,eta,D0,theta,Pbar,w,mu,H0,k,delta,r] = model1.params{:};
b    = -alpha;
a    = 1+b;
beta = 1/(1+r);

utility = @(p) a^2/(2*b)+(b*p.^2)/2-a*p-w*(p-Pbar).^2/2;

A    = s(:,1);

S0     = s0(:,1);
H0eps0 = s0(:,2);

S    = x0(:,1);
H    = x0(:,2);
P    = x0(:,3);

P0   = funeval(interp0.cx(:,3),interp0.fspace,(1-delta)*s0(:,1)+s0(:,2));

% Social welfare
W0t      = utility(P)+P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*S;
W00      = (1-beta)*mean(infinitesums(model0,interp0,s,x0,(1-delta)*s0(:,1)+s0(:,2),W0t));

% Producer surplus
PS0t  = beta*z0(:,2).*H-H0*H.^(1+mu)/(1+mu);
EPS00 = infinitesums(model0,interp0,s,x0,(1-delta)*s0(:,1)+s0(:,2),PS0t);
EPS00 = (1-beta)*mean(P0.*H0eps0+EPS00);

% Consumer utility before stabilisation
V0t = utility(P);
V00 = (1-beta)*mean(infinitesums(model0,interp0,s,x0,(1-delta)*s0(:,1)+s0(:,2),V0t));

Sg   = x1(:,1);
H    = x1(:,2);
P    = x1(:,3);
Xi   = x1(:,4);

P0   = funeval(interp1.cx(:,3),interp1.fspace,(1-delta)*s0(:,1)+s0(:,2));

% Social welfare
W1t      = utility(P)+P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*Sg;
W10      = (1-beta)*mean(infinitesums(model1,interp1,s,x1,(1-delta)*s0(:,1)+s0(:,2),W1t));

% Producer surplus
PS1t  = (beta*z1(:,2)+Xi).*H-H0*H.^(1+mu)/(1+mu);
EPS10 = infinitesums(model1,interp1,s,x1,(1-delta)*s0(:,1)+s0(:,2),PS1t);
EPS10 = (1-beta)*mean(P0.*H0eps0+EPS10);

% Fiscal cost
CostSt  = (P+k-beta*(1-delta)*z1(:,1)).*Sg;
ECostS0 = infinitesums(model1,interp1,s,x1,(1-delta)*s0(:,1)+s0(:,2),CostSt)-P0.*S0*(1-delta);
ECostS0 = (1-beta)*mean(ECostS0);
CostHt  = Xi.*H;
ECostH0 = (1-beta)*mean(infinitesums(model1,interp1,s,x1,(1-delta)*s0(:,1)+s0(:,2),CostHt));
ECost0 = ECostS0+ECostH0;

% Consumer utility after stabilisation
V1t = utility(P);
V10 = (1-beta)*mean(infinitesums(model1,interp1,s,x1,(1-delta)*s0(:,1)+s0(:,2),V1t));

EV = V10-V00;

disp('Welfare change:')
[EV; EPS10-EPS00; -ECost0; W10-W00]*100