function [W0,EPS0,ESS0,PA0] = Welfmodel(model,interp,s,x,z,s0)
% WELFMODEL Calculates the welfare effects on the benchmark model

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = model.params{:};
beta = 1/(1+r);

A    = s;

S    = x(:,1);
H    = x(:,2);
P    = x(:,3);

S0   = s0(:,1);
H0eps0 = s0(:,2);
P0   = funeval(interp.cx(:,3),interp.fspace,(1-delta)*s0(:,1)+s0(:,2));

%% Social welfare
utility = @(p,y) (y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;
Wt      = utility(P,Y)+w*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*S);
W0      = mean(infinitesums(model,interp,s,x,(1-delta)*s0(:,1)+s0(:,2),Wt));

%% Producer surplus
PSt  = beta*z(:,2).*H-H0*H.^(1+mu)/(1+mu);
EPS0 = infinitesums(model,interp,s,x,(1-delta)*s0(:,1)+s0(:,2),PSt);
EPS0 = (1-beta)*mean(P0.*H0eps0+EPS0);

%% Storer surplus
SSt  = (beta*(1-delta)*z(:,1)-P-k).*S;
ESS0 = infinitesums(model,interp,s,x,(1-delta)*s0(:,1)+s0(:,2),SSt);
ESS0 = (1-beta)*mean(P0.*S0*(1-delta)+ESS0);

%% Gains from first period grain suply
PA0  = (1-beta)*mean(P0.*(S0*(1-delta)+H0eps0));