function [X0,c] = infinitesums(model,interp,s,x,s0,Xt,c)
% INFINITESUMS Calculates terms of infinite sums by converting them into recursive
% equations

if nargin < 7, c = []; end

% Initialisation
e      = model.e;
func   = model.func;
params = model.params;
w      = model.w;

fspace = interp.fspace;
Phi    = interp.Phi;

options = struct(...
    'lmeth',3,...
    'showiters',0,...
    'atol',1E-10,...
    'rtol',eps);

r    = params{end};
beta = 1/(1+r);

n         = size(s,1);
K         = size(e,1);
ind       = (1:n);
ind       = ind(ones(1,K),:);
ss        = s(ind,:);
xx        = x(ind,:);
ee        = e(repmat(1:K,1,n),:);
snext     = feval(func,'g',ss,xx,[],ee,[],[],params{:});
Phinext   = funbasx(fspace,snext);

if isempty(c), c = funfitxy(fspace,Phi,Xt); end

EquationX = @(C) funfitxy(fspace,Phi,...
                          Xt+beta*reshape(w'*reshape(funeval(C,fspace,Phinext),K,n),n,1))-C;
c          = nsoli(EquationX,c,options);
X0         = funeval(c,fspace,s0);
