function [params,shocks,interp0,interp1,interp2] = init_common
% INIT_COMMON Initialises parameters

isParams = dir('Params.mat');
if ~isempty(isParams), load('Params.mat'); end

% Model parameters

%% Demand functions
%%% Commodity budget share
if ~exist('gamma','var'), gamma = 0.15; end
Y     = 1/gamma;              % Income
%%% Price elasticity of demand
if ~exist('alpha','var'), alpha = -0.4; end
%%% Income elasticity of demand
if ~exist('eta','var'), eta   = 0.5; end
D0    = gamma^eta;

vhat0 = Y^(1-eta)/(1-eta)-D0*1^(1+alpha)/(1+alpha);
% Fix theta find rho
%{
theta = -11.5;
rho   = eta-theta*Y^(1-eta)/vhat0;
%}
% Fix rho find theta
if ~exist('rho','var'), rho   = 2; end
theta = vhat0*(eta-rho)/(Y^(1-eta));

v0    = abs(vhat0.^(1+theta)/(1+theta));
fprintf(1,'Relative risk aversion: %1.3f \n',rho);

% Welfare weight parameter
isWeight = dir('Weight.mat');
if isempty(isWeight)
  w     = Y^(-eta).*vhat0.^(theta)/v0;
else
  load('Weight.mat');
end


%% Storage costs
if ~exist('k','var'), k = 0.06; end
delta = 0.02;
if ~exist('r','var'), r = 0.05; end

%% Supply functions
if ~exist('mu','var'), mu = 10; end
H0    = 1/(1+r);

params = {gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r};

% Shocks
A                   = 2;
B                   = 2;
[shocks.e,shocks.w] = qnwbeta(7,A,B);
emax                = 1.25;
emin                = 0.75;
shocks.e            = shocks.e*(emax-emin)+emin;
shocks.funrand      = @(nrep) betarnd(A,B,nrep,1)*(emax-emin)+emin;

sigma                 = sqrt(A*B/(A+B+1))/(A+B)*(emax-emin);
fprintf(1,'CV of yield: %1.3f \n',sigma);

% APPROXIMATION SPACE
%% Without policy (one state variable)
smin          = emin;
smax          = 2.1;
interp0.fspace = fundefn('spli',...
                         50,...
                         smin,...
                         smax);                 % function space
interp0.Phi    = funbasx(interp0.fspace);

%% With policy
order          = [50;    15];                                 % degree of approximation
% $$$ smin           = [emin; -0.05];
% $$$ smax           = [2.1;   0.15];
smin           = [emin; -0.005];
smax           = [2.1;   0.02];
interp1.fspace = fundefn('spli',...
                         order,...
                         smin,...
                         smax);                 % function space
interp1.Phi    = funbasx(interp1.fspace);

%% With policy
order          = [50;    15];                                 % degree of approximation
smin           = [emin; -5E-4];
smax           = [2.1;   5E-3];
interp2.fspace = fundefn('spli',...
                         order,...
                         smin,...
                         smax);                 % function space
interp2.Phi    = funbasx(interp2.fspace);
