function [out1,out2,out3,out4,out5] = mcommitmentsto(flag,s,x,z,e,snext,xnext, ...
                                                  gamma,Y,alpha,eta,D0,theta,v0, ...
                                                  w,mu,H0,k,delta,r)
% MCOMMITMENTSTO Model definition: optimal policy of public storage with private
% storage

n = size(s,1);
d = 2;
m = 9;
p = 4;

% s = [A X]
jA       = 1;
jX       = 2;
if ~isempty(s)
  A       = s(:,jA);
  X       = s(:,jX);
end

% x = [S H P Sg Mu Lambda Nu Delta Omega]
iS      = 1;
iH      = 2;
iP      = 3;
iSg     = 4;
iMu     = 5;
iNu     = 6;
iOmega  = 7;
iLambda = 8;
iDelta  = 9;

if ~isempty(x)
  S       = x(:,iS);
  Mu      = x(:,iMu);
  Sg      = x(:,iSg);
  H       = x(:,iH);
  P       = x(:,iP);
  Lambda  = x(:,iLambda);
  Nu      = x(:,iNu);
  Delta   = x(:,iDelta);
  Omega   = x(:,iOmega);
end

switch flag
 case 'f' % EQUILIBRIUM FUNCTION
  if nargout==1
    dvdp     = utility(P,Y);
    [D,dDdp] = demand(P,Y);
  else
    [D,dDdp,d2Ddp2] = demand(P,Y);
    [dvdp,d2vdp2]   = utility(P,Y);
  end

  % f
  out1            = zeros(n,m);
  out1(:,iSg)     = w*(P+k)+Omega-((1-delta)/(1+r))*z(:,3);
  out1(:,iS)      = out1(:,iSg)-Delta.*Mu;
  out1(:,iMu)     = -Lambda-Delta.*S;
  out1(:,iH)      = -Nu.*(mu*H0*H.^(mu-1))+z(:,4)/(1+r);
  out1(:,iP)      = dvdp-Omega.*dDdp+w*(A-Sg-S)-Lambda+X;
  out1(:,iLambda) = -P-k+Mu+((1-delta)/(1+r))*z(:,1);
  out1(:,iNu)     = z(:,2)/(1+r)-H0*H.^mu;
  out1(:,iDelta)  = Mu.*S;
  out1(:,iOmega)  = A-S-Sg-D;

  % df/dx
  if nargout>=2
    out2               = zeros(n,m,m);
    out2(:,iSg,iP)     = w*ones(n,1,1);
    out2(:,iSg,iOmega) = ones(n,1,1);
    out2(:,iS,:)       = out2(:,iSg,:);
    out2(:,iS,iMu)     = -Delta;
    out2(:,iS,iDelta)  = -Mu;
    out2(:,iMu,iS)     = -Delta;
    out2(:,iMu,iLambda) = -ones(n,1,1);
    out2(:,iMu,iDelta) = -S;
    out2(:,iH,iH)  = -Nu.*(mu*(mu-1)*H0*H.^(mu-2));
    out2(:,iH,iNu) = -mu*H0*H.^(mu-1);
    out2(:,iP,iP)      = d2vdp2-Omega.*d2Ddp2;
    out2(:,iP,iOmega)  = -dDdp;
    out2(:,iP,iS)      = -w*ones(n,1,1);
    out2(:,iP,iSg)     = -w*ones(n,1,1);
    out2(:,iP,iLambda) = -ones(n,1,1);
    out2(:,iLambda,iMu) = ones(n,1,1);
    out2(:,iLambda,iP)  = -ones(n,1,1);
    out2(:,iNu,iH) = -mu*H0*H.^(mu-1);
    out2(:,iDelta,iS)  = Mu;
    out2(:,iDelta,iMu) = S;
    out2(:,iOmega,iS)  = -ones(n,1,1);
    out2(:,iOmega,iSg) = -ones(n,1,1);
    out2(:,iOmega,iP)  = -dDdp;

    % df/dz
    if nargout>=3
      out3              = zeros(n,m,p);
      out3(:,iS,3)      = -ones(n,1,1)*((1-delta)/(1+r));
      out3(:,iSg,3)     = -ones(n,1,1)*((1-delta)/(1+r));
      out3(:,iH,4)      = ones(n,1,1)/(1+r);
      out3(:,iLambda,1) = ones(n,1,1)*((1-delta)/(1+r));
      out3(:,iNu,2)     = ones(n,1,1)/(1+r);
    end
  end
 case 'g' % STATE TRANSITION FUNCTION
  % g
  out1             = zeros(n,d);
  out1(:,jA)       = (1-delta)*(S+Sg)+H.*e;
  out1(:,jX)       = (1-delta)*Lambda+Nu.*e;

  % dg/dx
  if nargout>=2
    out2                     = zeros(n,d,m);
    out2(:,jA,iS)            = ones(n,1,1)*(1-delta);
    out2(:,jA,iSg)           = ones(n,1,1)*(1-delta);
    out2(:,jA,iH)            = e;
    out2(:,jX,iLambda)       = ones(n,1,1)*(1-delta);
    out2(:,jX,iNu)           = e;

  end
 case 'h' % EXPECTATION FUNCTION
  % h
  out1      = zeros(size(snext,1),p);
  out1(:,1) = xnext(:,iP);
  out1(:,2) = xnext(:,iP);
  out1(:,3) = w*xnext(:,iP)+xnext(:,iOmega);
  out1(:,4) = xnext(:,iOmega);

   % dh/dx
  if nargout>=2
    out2 = zeros(size(snext,1),p,m);

    % dh/dsnext
    if nargout>=3
      out3 = zeros(size(snext,1),p,d);

      % dh/dxnext
      if nargout>=4
        out4             = zeros(size(snext,1),p,m);
        out4(:,1,iP)     = ones(size(snext,1),1,1);
        out4(:,2,iP)     = ones(size(snext,1),1,1);
        out4(:,3,iP)     = w*ones(size(snext,1),1,1);
        out4(:,3,iOmega) = ones(size(snext,1),1,1);
        out4(:,4,iOmega) = ones(size(snext,1),1,1);

        % hmult
        if nargout==5
          out5        = ones(size(snext,1),p);
          out5(:,2) = e;
          out5(:,4) = e;
        end
      end
    end
  end
 case 'b' % BOUND FUNCTION
  out1 = [zeros(n,1) -inf(n,2) zeros(n,2) -inf(n,4)];
  out2 = inf(n,m);
end

function [dvdp,d2vdp2] = utility(p,y)

  vhat = y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha);

%  v    = vhat.^(1+theta)/(1+theta);
  dvdp = (-D0*p.^alpha.*vhat.^(theta))/v0;

  if nargout==2
    d2vdp2  = (theta*D0^2*p.^(2*alpha).*vhat.^(theta-1)-alpha*D0*p.^(alpha-1).*vhat.^theta)/v0;
  end
end

function [D,dDdp,d2Ddp2] = demand(p,y)

  D    = D0*p.^alpha.*y.^eta;
  dDdp = alpha*D./p;

  if nargout==3
    d2Ddp2  = (alpha-1)*dDdp./p;
  end
end

end