function [out1,out2,out3,out4,out5] = mfutures(flag,s,x,z,e,snext,xnext,...
                                               gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,...
                                               k,delta,r)
% MFUTURES Model with futures markets
  
d = 3;
m = 5;
p = 3;

iA  = 1;
iFs = 2;
iQs = 3;
if ~isempty(s)
  A  = s(:,iA);
  Fs = s(:,iFs);
  Qs = s(:,iQs);
end

% x = [S H P F]
iS = 1;
iH = 2;
iP = 3;
iF = 4;
iQ = 5;
if ~isempty(x)
  S  = x(:,iS);
  H  = x(:,iH);
  P  = x(:,iP);
  F  = x(:,iF);
  Q  = x(:,iQ);
end

switch flag
 case 'f' % EQUILIBRIUM FUNCTION
   n = size(s,1);
  if nargout >= 2
    [D,Dp,Dy] = demand(P,Y+(P-Fs).*Qs);
  else
    D = demand(P,Y+(P-Fs).*Qs);
  end

  % f
  out1       = zeros(n,m);
  out1(:,iS) = P+k-((1-delta)/(1+r))*z(:,1);
  out1(:,iH) = z(:,2)-(1+r)*H0*H.^mu;
  out1(:,iP) = A-D-S;
  out1(:,iF) = z(:,1)-F;
  out1(:,iQ) = z(:,3);

  % df/dx
  if nargout >= 2
    out2          = zeros(n,m,m);
    out2(:,iS,iP) = ones(n,1,1);
    
    out2(:,iH,iH) = -(1+r)*mu*H0*H.^(mu-1);
    
    out2(:,iP,iS) = -ones(n,1,1);
    out2(:,iP,iP) = -Dp-Qs.*Dy;
    
    out2(:,iF,iF) = -ones(n,1,1);

    % df/dz
    if nargout >= 3
      out3         = zeros(n,m,p);
      out3(:,iS,1) = -((1-delta)/(1+r))*ones(n,1,1);
      out3(:,iH,2) = ones(n,1,1);
      out3(:,iF,1) = ones(n,1,1);
      out3(:,iQ,3) = ones(n,1,1);
    end
  end

 case 'g' % STATE TRANSITION FUNCTION
   n = size(s,1);
  % g
  out1        = zeros(n,d);
  out1(:,iA)  = (1-delta)*S + H.*e;
  out1(:,iFs) = F;
  out1(:,iQs) = Q;

  % dg/dx
  if nargout>=2
    out2         = zeros(n,d,m);
    out2(:,iA,iS) = (1-delta)*ones(n,1,1);
    out2(:,iA,iH) = e;
    out2(:,iFs,iF) = ones(n,1,1);
    out2(:,iQs,iQ) = ones(n,1,1);
  end

 case 'h' % EXPECTATION FUNCTION
   [Vy,Vyy,Vyp] = utility(xnext(:,iP),Y+(xnext(:,iP)-snext(:,iFs)).*snext(:,iQs));

  % h
  out1      = zeros(size(snext,1),p);
  out1(:,1) = xnext(:,iP);
  out1(:,2) = xnext(:,iP);
  out1(:,3) = (xnext(:,iP)-snext(:,iFs)).*Vy;
  
  % dh/dx
  if nargout>=2
    out2         = zeros(size(snext,1),p,m);

    % dh/dsnext
    if nargout>=3
      out3          = zeros(size(snext,1),p,d);
      out3(:,3,iFs) = -Vy-(xnext(:,iP)-snext(:,iFs)).*snext(:,iQs).*Vyy;
      out3(:,3,iQs) = (xnext(:,iP)-snext(:,iFs)).^2.*Vyy;

      % dh/dxnext
      if nargout>=4
        out4         = zeros(size(snext,1),p,m);
        out4(:,1,iP) = ones(size(snext,1),1,1);
        out4(:,2,iP) = ones(size(snext,1),1,1);
        out4(:,3,iP) = Vy+(xnext(:,iP)-snext(:,iFs)).*(Vyp+snext(:,iQs).*Vyy);

        % hmult
        if nargout==5
          out5      = ones(size(snext,1),p);
          out5(:,2) = e;
        end
      end
    end
  end

 case 'b' % BOUND FUNCTION
   n = size(s,1);
  out1 = [zeros(n,1) -inf(n,4)];
  out2 = [A           inf(n,4)];

 case 'e' % EULER EQUATION ERROR
   out1 = [];
end

function [D,Dp,Dy] = demand(p,y)

  D = D0*p.^alpha.*y.^eta;
  if nargout>=2
    Dp = alpha*D./p; 
    Dy =   eta*D./y; 
  end

end

function [Vy,Vyy,Vyp] = utility(p,y)
  
  vhat = y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha);

%  v    = vhat.^(1+theta)/(1+theta);
%   dvdp = (-D0*p.^alpha.*vhat.^(theta))/v0;
  Vy = (y.^(-eta).*vhat.^(theta))/v0;

  if nargout>=2
    Vyy = (theta*y.^(-2*eta).*vhat.^(theta-1)-eta*y.^(-eta-1).*vhat.^theta)/v0;
    Vyp = (theta*y.^(-eta).*(-D0*p.^alpha).*vhat.^(theta-1))/v0;
  end

end

end