function [out1,out2,out3,out4] = mmodelwelf(flag,s,x,z,e,snext,xnext,...
                                            gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,...
                                            k,delta,r)
% MMODEL Model definition: a competitive storage model with supply reaction

n = size(s,1);
d = 1;
m = 4;
p = 3;

A = s;

% x = [S H P W]
iS = 1;
iH = 2;
iP = 3;
iW = 4
if ~isempty(x)
  S  = x(:,iS);
  H  = x(:,iH);
  P  = x(:,iP);
  W  = x(:,iW);
end

switch flag
 case 'f' % EQUILIBRIUM FUNCTION
  if nargout >= 2
    [D,dDdP] = demand(P,Y);
  else
    D = demand(P,Y);
  end
    [U,Up]   = utility(P);

  % f
  out1       = zeros(n,m);
  out1(:,iS) = P+k-((1-delta)/(1+r))*z(:,1);
  out1(:,iH) = z(:,2)-(1+r)*H0*H.^mu;
  out1(:,iP) = A-D-S;
  out1(:,iW) = U+w*(P.*A-H0*H.^(1+mu)/(1+mu)-(P+k).*S)+z(:,3)/(1+r);

  % df/dx
  if nargout >= 2
    out2          = zeros(n,m,m);
    out2(:,iS,iP) = ones(n,1,1);
    out2(:,iH,iH) = -(1+r)*mu*H0*H.^(mu-1);
    out2(:,iP,iS) = -ones(n,1,1);
    out2(:,iP,iP) = -dDdP;

    out2(:,iW,iP) = Up+w*(A-S);
    out2(:,iW,iH) = -w*H0*H.^mu;
    out2(:,iW,iS) = -w*(P+k);

    % df/dz
    if nargout >= 3
      out3         = zeros(n,m,p);
      out3(:,iS,1) = -((1-delta)/(1+r))*ones(n,1,1);
      out3(:,iH,2) = ones(n,1,1);
      
      out3(:,iW,3) = ones(n,1,1)/(1+r);
    end
  end

 case 'g' % STATE TRANSITION FUNCTION
  % g
  out1 = (1-delta)*S + H.*e;

  % dg/dx
  if nargout>=2
    out2         = zeros(n,d,m);
    out2(:,1,iS) = (1-delta)*ones(n,1,1);
    out2(:,1,iH) = e;
  end

 case 'h' % EXPECTATION FUNCTION
  % h
  out1      = zeros(size(snext,1),p);
  out1(:,1) = xnext(:,iP);
  out1(:,2) = e.*xnext(:,iP);
  out1(:,3) = xnext(:,iW);

  % dh/dx
  if nargout>=2
    out2 = zeros(size(snext,1),p,m);

    % dh/dsnext
    if nargout>=3
      out3 = zeros(size(snext,1),p,d);

      % dh/dxnext
      if nargout==4
        out4         = zeros(size(snext,1),p,m);
        out4(:,1,iP) = ones(size(snext,1),1,1);
        out4(:,2,iP) = e;
        out4(:,3,iW) = ones(size(snext,1),1,1);
      end
    end
  end

 case 'b' % BOUND FUNCTION
  out1 = [zeros(n,1) -inf(n,3)];
  out2 = [A inf(n,3)];

 case 'e' % EULER EQUATION ERROR
  out1       = zeros(n,2);
  out1(:,iS) = ones(n,1)-demand(max((A/(D0*Y^eta)).^(1/alpha),((1-delta)/(1+r))*z(:,1)-k),Y)./(A-S);
  out1(:,iH) = ones(n,1)-(((z(:,2)/(1+r))/H0).^(1/mu))./H;

end

function [D,dDdp] = demand(p,y)

  D = D0*p.^alpha.*y.^eta;
  if nargout==2, dDdp = alpha*D./p; end

end

function [U,Up] = utility(p)

  U  = (Y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^(1+theta)/(1+theta)/v0;
  Up = -D0*p.^alpha.*(Y.^(1-eta)/(1-eta)-D0*p.^(1+alpha)/(1+alpha)).^theta/v0;

end

end