function [model,interp,s,x,z] = runcommitmentfull(private)
% RUNCOMMITMENTFULL Run model: optimal policy

if nargin < 1, private = 1; end

% GET parameters
[params,shocks,interp0,interp] = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

% PACK MODEL STRUCTURE
model.func   = 'mcommitmentfull';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

options = struct(...
    'display'    , 1,...
    'reesolver'  , 'sa',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 1,...
    'loop_over_s', 0,...
    'reesolveroptions',struct('lambda',1,'atol',1E-10));

% Without private storage
n0 = prod(interp0.fspace.n);
s0 = gridmake(funnode(interp0.fspace));

options.reesolver = 'sa';

model0      = model;
model0.func = 'mcommitmentfullbis';
disp('Deterministic steady state:');
[sss,xss,zss] = recsSS(model0,1,[0 1 1 0 0],options)
% recsCheck(model0,sss,xss,zss);

xinit0  = [max(0,0.8*(s0(:,1)-1)) ones(n0,1) max(0.7,(s0(:,1)/(D0*Y^eta)).^(1/alpha)) ...
           zeros(n0,2)];
interp0.cx = funfitxy(interp0.fspace,interp0.Phi,xinit0);
interp0.ch = funfitxy(interp0.fspace,interp0.Phi,feval(model0.func,'h',[],[],[],[],s0,xinit0,model0.params{:}));

[interp0.cx,x0,z0] = recsSolveREE(interp0,model0,s0,xinit0,options);
interp0.cz = funfitxy(interp0.fspace,interp0.Phi,z0);

[LB,UB]          = feval(model0.func,'b',s(:,1),[],[],[],[],[],model0.params{:});
x                = zeros(n,9);
x(:,[4 2:3 6:7]) = min(max(funeval(interp0.cx,interp0.fspace,s(:,1)),LB),UB);
interp.cx        = funfitxy(interp.fspace,interp.Phi,x);

z         = funeval(interp0.cz,interp0.fspace,s(:,1));
interp.cz = funfitxy(interp.fspace,interp.Phi,z);
interp.ch = funfitxy(interp.fspace,interp.Phi,feval(model.func,'h',[],[],[],[],s,x,model.params{:}));

if private  % With private storage

  disp('Deterministic steady state:');
  [sss,xss,zss] = recsSS(model,[1 0],[0 1 1 0 0  0 0 0 0],options)

%  recsCheck(model,sss,xss,zss);

  options.reesolver = 'sa';
  options.reesolveroptions = struct('lambda',0.75,'atol',1E-10);
  [interp.cx,x,z] = recsSolveREE(interp,model,s,x,options);
  interp.cz = funfitxy(interp.fspace,interp.Phi,z);
end

sext0 = [0 1 0 0];
s0    = [1 0];

% Alternative welfare weight
disp('Welfare weight:');
disp(w);
vhat = Y^(1-eta)/(1-eta)-D0*x(:,3).^(1+alpha)/(1+alpha);
wt   = Y^(-eta).*vhat.^(theta)/v0;
disp('Welfare weight calculated in a CV approach:');
disp((1-1/(1+r))*infinitesums(model,interp,s,x,s0,wt))

[ssim,xsim,esim] = recsSimul(model,interp,repmat(s0,1E4,1),118,[],options);

[W0,EPS0,ECost0,ESS0,EV,ECostS0,ECostH0,Transfer] = Welffull(model,interp,s,x,z,sext0)

full = struct('model',model,...
              'interp',interp,...
              's',s,...
              'x',x,...
              'z',z);

save('models.mat','full','-append');
