function [model,interp,s,x,z] = runcommitmentsto(private)
% RUNCOMMITMENTSTO Run model: optimal public storage

if nargin < 1, private = 1; end

% GET parameters
[params,shocks,~,interp] = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

% PACK MODEL STRUCTURE
model.func   = 'mcommitmentsto';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

options = struct(...
    'display'    , 1,...
    'reesolver'  , 'sa',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 1,...
    'loop_over_s', 0,...
    'reesolveroptions',struct('lambda',0.75));

% Without private storage
model0      = model;
model0.func = 'mcommitmentstobis';

[sss,xss,zss] = recsSS(model0,[1 0],[0 1 1 0 0],options)
recsCheck(model0,sss,xss,zss);

options.reesolver = 'mixed';
options.reesolveroptions = struct('lambda',0.75);

load('models.mat','full');
xinit0        = zeros(n,5);
xinit0(:,[1:3 5]) = funeval(full.interp.cx(:,[4 2:3 7]),full.interp.fspace,s);
xinit0(:,1)       = max(xinit0(:,1),0);

interp.cx = funfitxy(interp.fspace,interp.Phi,xinit0);

[interp.cx,x0,z0] = recsSolveREE(interp,model0,s,xinit0,options);
interp.cz = funfitxy(interp.fspace,interp.Phi,z0);

x                = zeros(n,9);
x(:,[4 2:3 6:7]) = x0;
interp.cx        = funfitxy(interp.fspace,interp.Phi,x);

z = z0;

if private
  % With private storage
  disp('Deterministic steady state:');
  [sss,xss,zss] = recsSS(model,[1 0],[0 1 1 0 0  0 0 0 0],options)

  recsCheck(model,sss,xss,zss);

  options.method = 'expapprox';
  [interp.cz,x,z] = recsSolveREE(interp,model,s,x,options);
  interp.cx = funfitxy(interp.fspace,interp.Phi,x);
end

sext0 = [0 1 0 0];
s0    = [1 0];

[ssim,xsim,esim] = recsSimul(model,interp,repmat(s0,1E4,1),118,[],options);

CostS = (xsim(:,3,2:end)+k).*xsim(:,4,2:end)-xsim(:,3,2:end).*xsim(:,4,1:end-1);
mCostS = squeeze(mean(CostS,1));
figure()
plot(mCostS)

[W0,EPS0,ECost0,ESS0,EV,Transfer] = Welfsto(model,interp,s,x,z,sext0)

sto = struct('model',model,...
             'interp',interp,...
             's',s,...
             'x',x,...
             'z',z);

save('models.mat','sto','-append');

