function [model,interp,s,x,z] = runcommitmentsub
% RUNCOMMITMENTSUB Run model: optimal production subvention

% GET parameters
[params,shocks,~,interp] = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

% PACK MODEL STRUCTURE
model.func   = 'mcommitmentsub';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

xinit  = [max(0,0.8*(s(:,1)-1)) ones(n,1) max(0.7,(s(:,1)/(D0*Y^eta)).^(1/alpha)) zeros(n,5)];
interp.cz = funfitxy(interp.fspace,interp.Phi,ones(n,4));
interp.cx = funfitxy(interp.fspace,interp.Phi,xinit);
interp.ch = funfitxy(interp.fspace,interp.Phi,ones(n,4));

options = struct(...
    'display'    , 1,...
    'reesolver'  , 'sa',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 1,...
    'loop_over_s', 1,...
    'reesolveroptions',struct('lambda', 0.5,...
                              'atol'  , 1E-8));

disp('Deterministic steady state:');
[sss,xss,zss] = recsSS(model,[1 0],[0 1 1 0  0 0 0 0],options)

recsCheck(model,sss,xss,zss);

sext0 = [0 1 0];
s0    = [1 0];

[interp.cx,x,z] = recsSolveREE(interp,model,s,xinit,options);
interp.cz = funfitxy(interp.fspace,interp.Phi,z);

[ssim,xsim,esim] = recsSimul(model,interp,repmat(s0,1E4,1),118,[],options);

[W0,EPS0,ECost0,ESS0,EV,Transfer] = Welfsub(model,interp,s,x,z,sext0)

sub = struct('model',model,...
             'interp',interp,...
             's',s,...
             'x',x,...
             'z',z);

save('models.mat','sub','-append');
