function [model,interp,s,x,z] = runfutures

%% GET parameters
[params,shocks] = init_common;

interp.fspace = fundefn('spli',...
                        [30 8 8]',...
                        [0.75 0.75 0]',...
                        [1.5  1.15 0.85]');                 % function space
interp.Phi    = funbasx(interp.fspace);

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

%% PACK MODEL STRUCTURE
model.func   = 'mfutures';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

xinit     = [max(zeros(n,1),0.8*(s(:,1)-1)) ones(n,1) max((s(:,1)/(D0*Y^eta)).^(1/alpha),0.7) ...
            ones(n,1) ones(n,1)];
interp.cz = ones(n,2);
interp.cx = xinit;
interp.ch = funfitxy(interp.fspace,interp.Phi,feval(model.func,'h',[],[],[],[],s,xinit,params{:}));
options = struct(...
    'display'    , 1,...
    'reesolver'  , 'mixed',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 1,...
    'reesolveroptions',struct('atol',1E-10,'rtol',1E-10));

[sss,xss,zss] = recsSS(model,[1 1 0.75],[0 1 1 1 0.75])
recsCheck(model,sss,xss,zss);

tic
[interp.cx,x,z] = recsSolveREE(interp,model,s,xinit,options);
toc
interp.cz = funfitxy(interp.fspace,interp.Phi,z);

[ssim,xsim] = recsSimul(model,interp,sss(ones(1E4,1),:),118,[],options);

recsAccuracy(model,interp,ssim);

[W0,EPS0,ECost0,ESS0,EV,Transfer,PA0] = Welffutures(model,interp,s,x,z,[0 1 0 0])

futures = struct('model',model,...
                 'interp',interp,...
                 's',s,...
                 'x',x,...
                 'z',z);

save('models.mat','futures','-append');
