# GRAPH_MATLAB Makes all the figures of the paper

source("Graph_Functions.r")
source("itall.r")
require(R.matlab)

LineType <- c("solid","dashed","dotdash","11")

# Data
## Import data from Matlab
Rules <- readMat("../Matlab/Rules.mat")
Rules2 <- readMat("../Matlab/Rules2.mat")
Transition <- readMat("../Matlab/Transition.mat")
LongRun <- readMat("../Matlab/LongRun.mat")
Sample <- readMat("../Matlab/sample.mat")
acf2(t(Sample$xsim[1,,])[,1])[1:21,]

TransparentGrey <- rgb(red=0.4,green=0.4,blue=0.4)

xlim <- c(0.8,1.4)

# Plot
## Storage rules
Plot.SRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 1)
    plot(y    = rep(0,10),
         x    = seq(from=0.5,to=2,length.out=10),
         main = "",
         xlab = "Availability",
         ylab = "Stock",
         xaxs = "i",
         yaxs = "i",
         xaxt = "n",
         yaxt = "n",
         type = "l",
         ## col  = gray.colors(12)[3],
         col  = TransparentGrey,
         ylim = c(-0.01,0.25),
         xlim = xlim)
    lines(x   = ssim0,
          y   = xsim0[,1],
          lwd = 1.5)
    axis(side            = 2,
         at              = seq(from=0,to=0.7,by=0.1),
         labels          = c("0",seq(from=0.1,to=0.7,by=0.1)))
    axis(side            = 1,
         at              = seq(from=0.8,to=1.8,by=0.1),
         labels          = seq(from=0.8,to=1.8,by=0.1))
    lines(x   = ssim1[,1],
          y   = xsim1[,4],
          lty = LineType[2],
          lwd = 1.5)
    detach(Rules)
}
Plot.SRules()

## Features of storage rules (intercept and marginal propensity to store)
SRulesFeatures <- data.frame(matrix(NA,nrow=2,ncol=4),
                             row.names=c("Benchmark","Optimal"))
colnames(SRulesFeatures) <- c("Intercept","mps0","mps1","mps2")
attach(Rules)
i <- order(ssim0)
ssim0 <- ssim0[i]
xsim0 <- xsim0[i,]
SRulesFeatures[1,"Intercept"] <- ssim0[xsim0[,1]>0][1]
ds <- diff(ssim0)
dx <- diff(xsim0)
mps <- dx[,1]/ds
SRulesFeatures[1,2] <- mps[mps>0][2]
SRulesFeatures[1,3] <- mps[which.min(abs(ssim0-1.3))]
SRulesFeatures[1,4] <- mps[which.min(abs(ssim0-1.5))]

i <- order(ssim1[,1])
ssim1 <- ssim1[i,]
xsim1 <- xsim1[i,]
SRulesFeatures[2,"Intercept"] <- ssim1[xsim1[,4]>0,1][1]
ds <- diff(ssim1)
dx <- diff(xsim1)
mps <- dx[,4]/ds[,1]
SRulesFeatures[2,2] <- mps[mps>0][2]
SRulesFeatures[2,3] <- mps[which.min(abs(ssim1[,1]-1.3))]
SRulesFeatures[2,4] <- mps[which.min(abs(ssim1[,1]-1.5))]
detach(Rules)
SRulesFeatures

## Production rules
Plot.HRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 1)
    plot(x    = ssim1[,1],
         y    = xsim1[,2],
         main = "",
         xlab = "Availability",
         ylab = "Planned production",
         xaxs = "i",
         yaxs = "i",
         xaxt = "n",
         yaxt = "n",
         type = "l",
         lty = LineType[2],
         lwd  = 1.5,
         ## ylim = c(0.96,1.025),
         ylim = c(0.96,1.015),
         xlim = xlim)
    axis(side            = 2,
         at              = seq(from=0.96,to=1.04,by=0.01),
         labels          = seq(from=0.96,to=1.04,by=0.01))
    axis(side            = 1,
         at              = seq(from=0.8,to=1.8,by=0.1),
         labels          = seq(from=0.8,to=1.8,by=0.1))
    lines(x   = ssim0,
          y   = xsim0[,2],
          lty = LineType[1],
          lwd = 1.5)
    detach(Rules)
    legend(x             = "bottom",
           lty           = LineType[1:2],
           lwd           = 1.5,
           bty           = "n",
           legend        = c("Without public intervention",
           "Optimal stabilisation policy"))
}
Plot.HRules()

## Price rules
Plot.PRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 1)
    plot(x    = ssim0,
         y    = xsim0[,3],
         main = "",
         xlab = "Availability",
         ylab = "Price",
         xaxs = "i",
         yaxs = "i",
         xaxt = "n",
         yaxt = "n",
         type = "l",
         lty = LineType[1],
         lwd  = 1.5,
         ylim = c(0.6,1.8),
         xlim = xlim)
    axis(side            = 2,
         at              = seq(from=0.6,to=1.8,by=0.2),
         labels          = seq(from=0.6,to=1.8,by=0.2))
    axis(side            = 1,
         at              = seq(from=0.8,to=1.8,by=0.1),
         labels          = seq(from=0.8,to=1.8,by=0.1))
    lines(x   = ssim1[,1],
          y   = xsim1[,3],
          lty = LineType[2],
          lwd = 1.5)
    legend(x             = "topright",
           lty           = LineType[1:2],
           lwd           = 1.5,
           bty           = "n",
           legend        = c("Without public intervention",
           "Optimal stabilisation policy"))
    detach(Rules)
}
Plot.PRules()

## Subsidies rules
Plot.SubRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 1)
    plot(x    = ssim1[,1],
         y    = xsim1[,6],
         main = "",
         xlab = "Availability",
         ylab = "Subsidy/tax to planned production",
         xaxs = "i",
         xaxt = "n",
         yaxt = "n",
         type = "l",
         lty = LineType[2],
         lwd  = 1.5,
         ## ylim = c(-0.2,0.12),
         ylim = c(-0.1,0.05),
         xlim = xlim)
    abline(h=0,col=TransparentGrey)
    axis(side            = 2,
         at              = round(seq(from=-0.35,to=0.15,by=0.05),2),
         labels          = round(seq(from=-0.35,to=0.15,by=0.05),2))
    axis(side            = 1,
         at              = seq(from=0.8,to=1.8,by=0.1),
         labels          = seq(from=0.8,to=1.8,by=0.1))
    detach(Rules)
}
Plot.SubRules()

## Social marginal value
Plot.ChiRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 0)
    plot(x    = ssim1[,1],
         y    = xsim1[,7],
         main = "",
         xlab = "Availability",
         ylab = expression(paste("Social marginal value ",group("(",chi,")"))),
         xaxs = "i",
         yaxs = "i",
         xaxt = "n",
         yaxt = "n",
         type = "l",
         lty = LineType[2],
         lwd  = 1.5,
         ## ylim = c(-0.2,0.9),
         ylim = c(-0.025,0.1),
         xlim = xlim)
    abline(h=0,col=TransparentGrey)
    ## lines(x = Rules2$A,
    ##       y = Rules2$chi,
    ##       lwd = 1.5)
    ## axis(side            = 2,
    ##      at              = seq(from=-0.2,to=1,by=0.2),
    ##      labels          = round(seq(from=-0.2,to=1,by=0.2),1))
    axis(side            = 2,
         at              = seq(from=-0.2,to=1,by=0.025),
         labels          = round(seq(from=-0.2,to=1,by=0.025),3))
    axis(side            = 1,
         at              = seq(from=0.8,to=1.8,by=0.1),
         labels          = seq(from=0.8,to=1.8,by=0.1))
    detach(Rules)
}
Plot.ChiRules()

Plot.MuRules <- function() {
    attach(Rules)
    par(mar   = c(4.1,4.1,0.5,0.6),
        lwd   = 1,
        las   = 1)
    plot(xaxs = "i",
         xaxt = "n",
         yaxt = "n",
         main = "",
         xlab = "Availability",
         ylab = expression(paste("Marginal loss from storage ",group("(",phi,")"))),
         xlim = xlim,
         ylim = c(0,0.5),
         x    = ssim1[,1],
         y    = xsim1[,5],
         type = "l",
         lwd  = 1.5,
         lty = LineType[2])
    abline(h=0,col=TransparentGrey)
    lines(x    = Rules2$A,
          y    = Rules2$mu,
          lwd  = 1.5)
    axis(side            = 1,
         at              = seq(from=0.8,to=1.6,by=0.1),
         labels          = seq(from=0.8,to=1.6,by=0.1))
    axis(side            = 2,
         at              = seq(from=0,to=0.6,by=0.1),
         labels          = seq(from=0,to=0.6,by=0.1))
    detach(Rules)
    ## legend(x             = "topleft",
    ##        lty           = LineType[1:2],
    ##        lwd           = 1.5,
    ##        bty           = "n",
    ##        legend        = c("Without public intervention",
    ##        "Optimal stabilisation policy"))
}
Plot.MuRules()

FiguresCreation(file=c("../TeX/Figures/","SRules"),
                quote(Plot.SRules),width=3,height=3,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","HRules"),
                quote(Plot.HRules),width=3,height=3,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","PRules"),
                quote(Plot.PRules),width=3,height=3,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","SubRules"),
                quote(Plot.SubRules),width=3,height=3,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","ChiRules"),
                quote(Plot.ChiRules),width=3,height=3,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","MuRules"),
                quote(Plot.MuRules),width=3,height=3,pointsize=10)

## Transtional dynamics
Plot.ES <- function() {
    par(mar      = c(4.1,4.1,0.5,0.5),
        yaxt     = "s",
        las      = 1,
        lwd      = 1)
    matplot(x    = -1:(dim(Transition$mS)[1]-2),
            y    = Transition$mS[,c(2,3,1)],
            main = "",
            xlab = "Year",
            ylab = "Mean stock",
            type = "l",
            lty  = LineType[4:2],
            lwd  = 1.5,
            col  = "black",
            xaxs = "i",
            xlim = c(-1,15))
    abline(h   = Transition$mS[1],
           lty = LineType[1],
           lwd = 1.5)
    legend(x      = "right",
           inset  = c(0,0.05),
           lty    = LineType[c(1,4:2)],
           lwd    = 1.5,
           cex    = 0.9,
           bty    = "n",
           legend = c("Without public intervention",
           "Public storage",
           "Subsidy/tax to production",
           "Both instruments"))
}
Plot.ES()

Plot.EP <- function() {
    par(mar      = c(4.1,4.1,0.5,0.5),
        yaxt     = "s",
        las      = 1,
        lwd      = 1)
    matplot(x    = -1:(dim(Transition$mP)[1]-2),
            y    = Transition$mP[,c(2,3,1)],
            main = "",
            xlab = "Year",
            ylab = "Mean price",
            type = "l",
            lty  = LineType[4:2],
            col  = "black",
            lwd  = 1.5,
            xaxs = "i",
            xlim = c(-1,15))
    abline(h   = Transition$mP[1],
           lty = LineType[1],
           lwd = 1.5)
}
Plot.EP()

## Long-run costs
tail(Transition$meanCosts)

size = 3;
FiguresCreation(file=c("../TeX/Figures/","ES"),quote(Plot.ES),width=size,height=size,pointsize=10)
FiguresCreation(file=c("../TeX/Figures/","EP"),quote(Plot.EP),width=size,height=size,pointsize=10)

## Density
### Availability density
A.density <- function(x) density(x,adjust = 1.5,kernel="e",from=0.75)
Plot.A.density <- function() {
    par(mar   = c(4.1,2.1,0.5,0.6),
        yaxt  = "n",
        lwd   = 1)
    plot(A.density(LongRun$A[,1]),
         main = "",
         xlab = "Availability",
         ylab = "",
         xaxs = "i",
         yaxs = "i",
         xaxt = "n",
         ## xlim = c(0.75,1.7),
         xlim = c(0.75,1.5),
         ylim = range(A.density(LongRun$A[,4])$y)*c(1,1.02),
         lwd  = 1.5,
         zero.line = F)
    mtext(text = "Density",
          side = 2,
          line = 1)
    axis(side   = 1,
         at     = seq(from=0.8,to=1.6,by=0.1),
         labels = seq(from=0.8,to=1.6,by=0.1))
    lines(A.density(LongRun$A[,2]),
          lty = LineType[2],
          lwd = 1.5)
    lines(A.density(LongRun$A[,3]),
          lty = LineType[4],
          lwd = 1.5)
    lines(A.density(LongRun$A[,4]),
          lty = LineType[3],
          lwd = 1.5)
    legend(x             = "topright",
           lty           = LineType[c(1,4:3,2)],
           cex           = 1,
           lwd           = 1.5,
           bty           = "n",
           legend        = c("Without public intervention",
           "Public storage",
           "Subsidy/tax to production",
           "Both instruments"))
}
Plot.A.density()

FiguresCreation(file=c("../TeX/Figures/","ADensity"),quote(Plot.A.density),
                width=6,height=3,pointsize=10)


